package com.mysql.cluster.ndbj;

import com.mysql.cluster.ndbj.NdbError.Classification;
import com.mysql.cluster.ndbj.NdbError.Status;
import java.sql.SQLException;

/**
 * A NdbApiException is the base checked exception class of the NDB/J API Exception hierarchy.
 * <br>A NdbApiException is thrown if the error is generated in the cluster 
 * (and can either be recovered from or requires closing some open objects). 
 * It contains a message field, an error code field and a status field describing
 * the cause of the problem.
 * <br>If the error is generated by NDB/J (rather than internally in the cluster);
 * then, an error message is supplied (but no error code or error status from the cluster).
 * @see NdbError
 */
public class NdbApiException extends SQLException
{

    protected static final long serialVersionUID=1L;
    protected final String message;
    protected final NdbError errRef; 

    /**
     * constructor for a NdbApiException can be created using a native
     * NdbError object.
     * @param error
     */
    public NdbApiException(long errPtr)
	{
    	super();
    	errRef = new NdbError(errPtr,true);
    	this.message=errRef.getMessage();
;
	}


    /**
     * constructor for a NdbApiException that uses a String to describe
     * the condition that caused the exception.
     * @param message
     */
    public NdbApiException(String message)
	{
    	super(message);
	    this.message=message;
	    this.errRef=null;
	}

    public NdbError getErrorObj() { 
    	return this.errRef;
    }
    /**
     * @return String a stringified message describing the cause of the exception.
     */
    @Override
    public String toString() {
    	//if (errRef == null) { 
    		return message;
    	//}
        //return "ERROR: " + errRef.getCode() + " Message: " + message
        //+ " Classification: " + errRef.getClassification();
    }

    /**
     * @return String a stringified message describing the cause of the exception.
     */
    @Override
    public String getMessage() {
    	return toString();
    }


	/**
	 * @return
	 * @see com.mysql.cluster.ndbj.NdbError#getClassification()
	 */
	public Classification getClassification() {
		if (errRef != null) { 
			return errRef.getClassification();
		}
		return null;
	}


	/**
	 * @return
	 * @see com.mysql.cluster.ndbj.NdbError#getCode()
	 */
	public int getCode() {
    	if (errRef == null) { 
    		return -1;
    	}
		return errRef.getCode();
	}


	/**
	 * @return
	 * @see com.mysql.cluster.ndbj.NdbError#getStatus()
	 */
	public Status getStatus() {
		return errRef.getStatus();
	}

}
