package com.mysql.cluster.ndbj;

import java.util.Map;


/**
 * NdbFactory is a class with a public static factory method for creating
 * an NdbClusterConnection object. NdbFactory also ensures that the JNI 
 * library is loaded and initialized. 
 * 
 */
public class NdbFactory
{
		

  static {
    try {
      System.loadLibrary("ndbj");
    } catch (UnsatisfiedLinkError e) {
      throw new RuntimeException("Native code library failed to load. \n" + e);
    }
    ndbj.ndb_init();
  }

    /**
     * One of the first methods typically called in a NDB/J application. 
     * <br>It is a factory method used to create the top-level NdbClusterConnection object.
     * <br>Example Usage:
     * <code>
     *      <br>NdbClusterConnection conn=null;
     *      <br>try {
     *          <br>conn = NdbFactory.createNdbClusterConnection("ndb-connectstring");
     *          <br>// use the NdbClusterConnection object
     *      <br>}
     *      <br>catch (NdbClusterConnectionPermanentException e) {
     *      <br>// Serious problem. Retrying connecting will not work.
     *      <br>}
     * </code>
     * @param connectString the "--connectstring" used to connect to the cluster's management server (ndb_mgmd) 
     * @return a NdbClusterConnectionImpl object
     * @throws NdbApiException if a null string is passed in, or a connection object could not be created successfully.
     */
    static public NdbClusterConnection createNdbClusterConnection(String connectString)
    throws NdbApiException
    {
        return new NdbClusterConnectionImpl(connectString);
    }

    /**
     * One of the first methods typically called in a NDB/J application. 
     * <br>It is a factory method used to create the top-level NdbClusterConnection object.
     * <br>This form, without an argument, lets the connect string default to localhost.
     * <br>Example Usage:
     * <code>
     *      <br>NdbClusterConnection conn=null;
     *      <br>try {
     *          <br>conn = NdbFactory.createNdbClusterConnection();
     *          <br>// use the NdbClusterConnection object
     *      <br>}
     *      <br>catch (NdbClusterConnectionPermanentException e) {
     *      <br>// Serious problem. Retrying connecting will not work.
     *      <br>}
     * </code>
     * @return a NdbClusterConnectionImpl object
     * @throws NdbApiException if a null string is passed in, or a connection object could not be created successfully.
     */
    static public NdbClusterConnection createNdbClusterConnection()
    throws NdbApiException
    {
        return new NdbClusterConnectionImpl(null);
        
    }

 

    /**
     * Used internally by NDB/J.
     * @param op
     * @param ht
     * @return
     */
    static NdbResultSet createNdbResultSet(NdbOperation op, Map<String,NdbRecAttr> ht)
    {
        return new NdbResultSetImpl(op, ht);
    }    
    
    /**
     * Used internally by NDB/J.
     * @param op
     * @param ht
     * @return
     */
    static NdbResultSet createNdbResultSet(NdbIndexScanOperation op, Map<String,NdbRecAttr> ht)
    {
        return new NdbResultSetImpl(op, ht);
        
    }    
    
    /**
     * Used internally by NDB/J.
     * @param op
     * @param ht
     * @return
     */
    static NdbResultSet createNdbResultSet(NdbScanOperation op, Map<String,NdbRecAttr> ht)
    {
        return new NdbResultSetImpl(op, ht);
       
    }        
 
}


