package com.mysql.cluster.ndbj;

import com.mysql.cluster.ndbj.NdbScanOperation.ScanFlag;

public interface NdbOperationFactory {

	/** 
	 * Set the LockMode for subsequently created Operations. Note - This does not set a global LockMode 
	 * for the transaction - it merely sets the lock mode to use for the next Operations created by this 
	 * OperationFactory
	 * @param theLockMode the LockMode to use
	 */
	public abstract void setOperationLockMode(NdbOperation.LockMode theLockMode); 
	
	/**
	 * Set the parallel flag for subsequently created Operations. 
	 * @param parallel Number of fragments to scan in parallel (0=max) 
	 */
	public abstract void setParallelFlag(long parallel); 
	
	/**
	 * Set the number of rows to return in each batch for subsequently created Operations
	 * @param batch Number of rows to fetch in each batch
	 */
	
	public abstract void setBatchFlag(long batch);
	/**
	 * Creates a NdbOperation (primary key operation) and adds it to the transaction.
	 * @param tableName the table to be operated upon
	 * @param theLockMode the lock mode to use
	 * @return an NdbOperation object
	 * @throws NdbApiException if tableName is null or empty, the object has been closed, or the 
	 * Operation object could not be created
	 */
	public abstract NdbOperation getSelectOperation(String tableName, NdbOperation.LockMode theLockMode) 
	throws NdbApiException;
	
	/**
	 * Creates a NdbOperation (primary key operation) and adds it to the transaction.
	 * @param tableName the table to be operated upon
	 * @return an NdbOperation object
	 * @throws NdbApiException if tableName is null or empty, the object has been closed, or the 
	 * Operation object could not be created
	 */
	public abstract NdbOperation getSelectOperation(String tableName) 
	throws NdbApiException;

	/**
	 * Creates a NdbOperation (primary key operation) and adds it to the transaction.
	 * @param aTable the table to be operated upon
	 * @param theLockMode the lock mode to use
	 * @return an NdbOperation object
	 * @throws NdbApiException if tableName is null or empty, the object has been closed, or the Operation object could not be created
	 */
	public abstract NdbOperation getSelectOperation(NdbTable table, NdbOperation.LockMode theLockMode) 
	throws NdbApiException;

	/**
	 * Creates a NdbOperation (primary key operation) and adds it to the transaction.
	 * @param aTable the table to be operated upon
	 * @return an NdbOperation object
	 * @throws NdbApiException if tableName is null or empty, the object has been closed, or the Operation object could not be created
	 */
	public abstract NdbOperation getSelectOperation(NdbTable table) 
	throws NdbApiException;
	
	/**
	 * Creates a NdbOperation (primary key operation) and adds it to the transaction.
	 * @param aTable the table to be operated upon
	 * @return an NdbOperation object
	 * @throws NdbApiException if tableName is null or empty, the object has been closed, or the Operation object could not be created
	 */
	public abstract NdbOperation getUpdateOperation(String tableName)
			throws NdbApiException;

	/**
	 * Creates a NdbOperation (primary key operation) and adds it to the transaction.
	 * @param aTable the table to be operated upon
	 * @return an NdbOperation object
	 * @throws NdbApiException if tableName is null or empty, the object has been closed, or the Operation object could not be created
	 */
	public abstract NdbOperation getUpdateOperation(NdbTable table)
			throws NdbApiException;

	/**
	 * Creates a NdbOperation (primary key operation) and adds it to the transaction.
	 * @param aTable the table to be operated upon
	 * @return an NdbOperation object
	 * @throws NdbApiException if tableName is null or empty, the object has been closed, or the Operation object could not be created
	 */
	public abstract NdbOperation getInsertOperation(String tableName)
			throws NdbApiException;

	/**
	 * Creates a NdbOperation (primary key operation) and adds it to the transaction.
	 * @param aTable the table to be operated upon
	 * @return an NdbOperation object
	 * @throws NdbApiException if tableName is null or empty, the object has been closed, or the Operation object could not be created
	 */
	public abstract NdbOperation getInsertOperation(NdbTable table)
			throws NdbApiException;

	/**
	 * Creates a NdbOperation (primary key operation) and adds it to the transaction.
	 * @param aTable the table to be operated upon
	 * @return an NdbOperation object
	 * @throws NdbApiException if tableName is null or empty, the object has been closed, or the Operation object could not be created
	 */
	public abstract NdbOperation getDeleteOperation(String tableName)
			throws NdbApiException;

	/**
	 * Creates a NdbOperation (primary key operation) and adds it to the transaction.
	 * @param aTable the table to be operated upon
	 * @return an NdbOperation object
	 * @throws NdbApiException if tableName is null or empty, the object has been closed, or the Operation object could not be created
	 */
	public abstract NdbOperation getDeleteOperation(NdbTable table)
			throws NdbApiException;

	/**
	 * Creates a NdbIndexOperation (secondary unique index operation) and adds it to the transaction.
	 * @param indexName index to be used
	 * @param tableName the table to be operated upon
	 * @param theLockMode the LockMode to use
	 * @return an NdbOperation object
	 * @throws NdbApiException if tableName is null or empty, the object has been closed, or the Operation object could not be created
	 */
	public abstract NdbIndexOperation getSelectUniqueOperation(String indexName, String tableName,
			NdbOperation.LockMode theLockMode) 
	throws NdbApiException;
	
	/**
	 * Creates a NdbIndexOperation (secondary unique index operation) and adds it to the transaction.
	 * @param indexName the index to use
	 * @param tableName the table to be operated upon
	 * @return an NdbOperation object
	 * @throws NdbApiException if tableName is null or empty, the object has been closed, or the Operation object could not be created
	 */
	public abstract NdbIndexOperation getSelectUniqueOperation(String indexName, String tableName) 
	throws NdbApiException;

	/**
	 * Creates a NdbIndexOperation (secondary unique index operation) and adds it to the transaction.
	 * @param index the index to use 
	 * @param table the table to be operated upon
	 * @param theLockMode the lock mode to use
	 * @return an NdbOperation object
	 * @throws NdbApiException if tableName is null or empty, the object has been closed, or the Operation object could not be created
	 */
	public abstract NdbIndexOperation getSelectUniqueOperation(NdbIndex index, NdbTable table,
			NdbOperation.LockMode theLockMode) throws NdbApiException;

	/**
	 * Creates a NdbIndexOperation (secondary unique index operation) and adds it to the transaction.
	 * @param index the index to use 
	 * @param table the table to be operated upon
	 * @return an NdbOperation object
	 * @throws NdbApiException if tableName is null or empty, the object has been closed, or the Operation object could not be created
	 */
	public abstract NdbIndexOperation getSelectUniqueOperation(NdbIndex index, NdbTable table) 
	throws NdbApiException;
	/**
	 * Creates a NdbIndexOperation (secondary unique index operation) and adds it to the transaction.
	 * @param aTable the table to be operated upon
	 * @return an NdbOperation object
	 * @throws NdbApiException if tableName is null or empty, the object has been closed, or the Operation object could not be created
	 */
	public abstract NdbIndexOperation getUpdateUniqueOperation(String indexName, String tableName)
			throws NdbApiException;

	/**
	 * Creates a NdbIndexOperation (secondary unique index operation) and adds it to the transaction.
	 * @param aTable the table to be operated upon
	 * @return an NdbOperation object
	 * @throws NdbApiException if tableName is null or empty, the object has been closed, or the Operation object could not be created
	 */
	public abstract NdbIndexOperation getUpdateUniqueOperation(NdbIndex index, NdbTable table)
			throws NdbApiException;


	/**
	 * Creates a NdbIndexOperation (secondary unique index operation) and adds it to the transaction.
	 * @param aTable the table to be operated upon
	 * @return an NdbOperation object
	 * @throws NdbApiException if tableName is null or empty, the object has been closed, or the Operation object could not be created
	 */
	public abstract NdbIndexOperation getDeleteUniqueOperation(String indexName, String tableName)
			throws NdbApiException;

	/**
	 * Creates a NdbIndexOperation (primary key operation) and adds it to the transaction.
	 * @param aTable the table to be operated upon
	 * @return an NdbOperation object
	 * @throws NdbApiException if tableName is null or empty, the object has been closed, or the Operation object could not be created
	 */
	public abstract NdbIndexOperation getDeleteUniqueOperation(NdbIndex index, NdbTable table)
			throws NdbApiException;
	
	/**
	 * Creates a NdbIndexOperation (primary key operation) and adds it to the transaction.
	 * @param aTable the table to be operated upon
	 * @return an NdbOperation object
	 * @throws NdbApiException if tableName is null or empty, the object has been closed, or the Operation object could not be created
	 */
	public abstract NdbIndexOperation getDeleteUniqueOperation(NdbIndex index)
			throws NdbApiException;
	
	/**
	 * Creates a NdbScanOperation (table scan operation) and adds it to the transaction.
	 * @param aTable the table to be operated upon
	 * @return an NdbScanOperation object
	 * @throws NdbApiException if tableName is null or empty, the object has been closed, or the Operation object could not be created
	 */
	public NdbScanOperation getSelectScanOperation(String tableName) 
	throws NdbApiException;
	
	/**
	 * Creates a NdbScanOperation (table scan operation) and adds it to the transaction.
	 * @param aTable the table to be operated upon
	 * @return an NdbScanOperation object
	 * @throws NdbApiException if tableName is null or empty, the object has been closed, or the Operation object could not be created
	 */
	public NdbScanOperation getSelectScanOperation(NdbTable table) 
	throws NdbApiException;
	
	/**
	 * Creates a NdbScanOperation (table scan operation) and adds it to the transaction.
	 * @param aTable the table to be operated upon
	 * @param scanFlag flag controlling the operation of the scan
	 * @return an NdbScanOperation object
	 * @throws NdbApiException if tableName is null or empty, the object has been closed, or the Operation object could not be created
	 */
	public NdbScanOperation getSelectScanOperation(String tableName, ScanFlag scanFlag) 
	throws NdbApiException;
	
	/**
	 * Creates a NdbScanOperation (table scan operation) and adds it to the transaction.
	 * @param aTable the table to be operated upon
	 * @param scanFlag flag controlling the operation of the scan
	 * @return an NdbScanOperation object
	 * @throws NdbApiException if tableName is null or empty, the object has been closed, or the Operation object could not be created
	 */
	public NdbScanOperation getSelectScanOperation(NdbTable table, ScanFlag scanFlag)
	throws NdbApiException;

	/**
	 * Creates a NdbScanOperation (table scan operation) and adds it to the transaction.
	 * @param aTable the table to be operated upon
	 * @param scanFlag OR'd set of ScanFlags controlling the operation of the scan
	 * @return an NdbScanOperation object
	 * @throws NdbApiException if tableName is null or empty, the object has been closed, or the Operation object could not be created
	 */
	public NdbScanOperation getSelectScanOperation(NdbTable table, long scanFlag)
	throws NdbApiException;
	
	/**
	 * Creates a NdbScanOperation (table scan operation) and adds it to the transaction.
	 * @param aTable the table to be operated upon
	 * @param scanFlag OR'd set of ScanFlags controlling the operation of the scan
	 * @return an NdbScanOperation object
	 * @throws NdbApiException if tableName is null or empty, the object has been closed, or the Operation object could not be created
	 */
	public NdbScanOperation getSelectScanOperation(String tableName, long scanFlag)
	throws NdbApiException;

	/**
	 * Creates a NdbIndexScanOperation (index scan operation) and adds it to the transaction.
	 * @param indexName the index to be scanned
	 * @param tableName the table to be operated upon
	 * @return an NdbIndexScanOperation object
	 * @throws NdbApiException if tableName is null or empty, the object has been closed, or the Operation object could not be created
	 */
	public NdbIndexScanOperation getSelectIndexScanOperation(String indexName, String tableName) 
	throws NdbApiException;
	
	/**
	 * Creates an NdbIndexScanOperation (index scan operation) and adds it to the transaction.
	 * @param index the index to be scanned
	 * @param table the table to be operated upon
	 * @return an NdbScanOperation object
	 * @throws NdbApiException if tableName is null or empty, the object has been closed, or the Operation object could not be created
	 */
	public NdbIndexScanOperation getSelectIndexScanOperation(NdbIndex index, NdbTable table) 
	throws NdbApiException;
	
	/**
	 * Creates a NdbIndexScanOperation (Index scan operation) and adds it to the transaction.
	 * @param index the index to be scanned
	 * @return an NdbScanOperation object
	 * @throws NdbApiException if tableName is null or empty, the object has been closed, or the Operation object could not be created
	 */
	public NdbIndexScanOperation getSelectIndexScanOperation(NdbIndex index) 
	throws NdbApiException;
	
	/**
	 * Creates a NdbIndexScanOperation (index scan operation) and adds it to the transaction.
	 * @param indexName the index to be scanned
	 * @param tableName the table to be operated upon
	 * @param scanFlag flag controlling the operation of the scan
	 * @return an NdbIndexScanOperation object
	 * @throws NdbApiException if tableName is null or empty, the object has been closed, or the Operation object could not be created
	 */
	public NdbIndexScanOperation getSelectIndexScanOperation(String indexName, String tableName, ScanFlag scanFlag) 
	throws NdbApiException;
	
	/**
	 * Creates an NdbIndexScanOperation (index scan operation) and adds it to the transaction.
	 * @param index the index to be scanned
	 * @param table the table to be operated upon
	 * @param scanFlag flag controlling the operation of the scan
	 * @return an NdbScanOperation object
	 * @throws NdbApiException if tableName is null or empty, the object has been closed, or the Operation object could not be created
	 */
	public NdbIndexScanOperation getSelectIndexScanOperation(NdbIndex index, NdbTable table, ScanFlag scanFlag)
	throws NdbApiException;
	
	/**
	 * Creates a NdbIndexScanOperation (Index scan operation) and adds it to the transaction.
	 * @param index the index to be scanned
	 * @param scanFlag flag controlling the operation of the scan
	 * @return an NdbScanOperation object
	 * @throws NdbApiException if tableName is null or empty, the object has been closed, or the Operation object could not be created
	 */
	public NdbIndexScanOperation getSelectIndexScanOperation(NdbIndex index, ScanFlag scanFlag)
	throws NdbApiException;
	
	/**
	 * Creates a NdbIndexScanOperation (index scan operation) and adds it to the transaction.
	 * @param indexName the index to be scanned
	 * @param tableName the table to be operated upon
	 * @param scanFlag OR'd set of ScanFlags controlling the operation of the scan
	 * @return an NdbIndexScanOperation object
	 * @throws NdbApiException if tableName is null or empty, the object has been closed, or the Operation object could not be created
	 */
	public NdbIndexScanOperation getSelectIndexScanOperation(String indexName, String tableName, long scanFlag) 
	throws NdbApiException;
	
	/**
	 * Creates an NdbIndexScanOperation (index scan operation) and adds it to the transaction.
	 * @param index the index to be scanned
	 * @param table the table to be operated upon
	 * @param scanFlag OR'd set of ScanFlags controlling the operation of the scan
	 * @return an NdbScanOperation object
	 * @throws NdbApiException if tableName is null or empty, the object has been closed, or the Operation object could not be created
	 */
	public NdbIndexScanOperation getSelectIndexScanOperation(NdbIndex index, NdbTable table, long scanFlag)
	throws NdbApiException;
	
	/**
	 * Creates a NdbIndexScanOperation (Index scan operation) and adds it to the transaction.
	 * @param index the index to be scanned
	 * @param scanFlag OR'd set of ScanFlags controlling the operation of the scan
	 * @return an NdbScanOperation object
	 * @throws NdbApiException if tableName is null or empty, the object has been closed, or the Operation object could not be created
	 */
	public NdbIndexScanOperation getSelectIndexScanOperation(NdbIndex index, long scanFlag)
	throws NdbApiException;
	
}
