package com.mysql.cluster.ndbj;


import java.sql.Timestamp;

/**
 * NdbResultSet is familiar to programmers who have used <a href="http://java.sun.com/j2se/1.5.0/docs/api/java/sql/ResultSet.html">ResultSet</a> from from JDBC.
 * <p>A NdbResultSet is used to store attributes in rows
 * that were selected after executing some query operation
 * on the cluster.
 * <br>Important: Id counts start at '0' (for the first column in a table). 
 * <br>Be consistent in use of NdbResultSet objects.
 * Either access columns by name (String) or id (int).
 * Do not mix usage of access by name and id.
 */
public interface NdbResultSet extends java.sql.ResultSet 
{

    /**
     * Get an Integer value the next entry in the NdbResultSet object. 
     * @param columnId column number (starting from '0')
     * @return integer value
     * @throws NdbApiException if a value of this type cannot be retrieved as the next entry in the NdbResultSet
     */
    public abstract int getInt(int columnId) throws NdbApiException;

    /**
     * Get an Integer value the next entry in the NdbResultSet object. 
     * @param columnName
     * @return integer value
     * @throws NdbApiException if a value of this type cannot be retrieved as the next entry in the NdbResultSet
     */
    public abstract int getInt(String columnName) throws NdbApiException;

    /**
     * Get a short value the next entry in the NdbResultSet object. 
     * @param columnId column number (starting from '0')
     * @return short value
     * @throws NdbApiException if a value of this type cannot be retrieved as the next entry in the NdbResultSet
     */
    public abstract short getShort(int columnId) throws NdbApiException;

    /**
     * Get a short value the next entry in the NdbResultSet object. 
     * @param columnName
     * @return short value
     * @throws NdbApiException if a value of this type cannot be retrieved as the next entry in the NdbResultSet
     */
    public abstract short getShort(String columnName) throws NdbApiException;

    /**
     * Get a short value the next entry in the NdbResultSet object. 
     * @param columnId column number (starting from '0')
     * @return long value
     * @throws NdbApiException if a value of this type cannot be retrieved as the next entry in the NdbResultSet
     */
    public abstract long getLong(int columnId) throws NdbApiException;

    /**
     *
     * @param columnName
     * @return long value
     * @throws NdbApiException if a value of this type cannot be retrieved as the next entry in the NdbResultSet
     */
    public abstract long getLong(String columnName) throws NdbApiException;

    /**
     *
     * @param columnId column number (starting from '0')
     * @return float value
     * @throws NdbApiException if a value of this type cannot be retrieved as the next entry in the NdbResultSet
     */
    public abstract float getFloat(int columnId) throws NdbApiException;

    /**
     *
     * @param columnName
     * @return float value
     * @throws NdbApiException if a value of this type cannot be retrieved as the next entry in the NdbResultSet
     */
    public abstract float getFloat(String columnName) throws NdbApiException;

    /**
     *
     * @param columnId column number (starting from '0')
     * @return double value
     * @throws NdbApiException if a value of this type cannot be retrieved as the next entry in the NdbResultSet
     */
    public abstract double getDouble(int columnId) throws NdbApiException;

    /**
     *
     * @param columnName
     * @return double value
     * @throws NdbApiException if a value of this type cannot be retrieved as the next entry in the NdbResultSet
     */
    public abstract double getDouble(String columnName) throws NdbApiException;

    /**
     *
     * @param columnId column number (starting from '0')
     * @return a Timestamp object
     * @throws NdbApiException if a value of this type cannot be retrieved as the next entry in the NdbResultSet
     */
    public abstract Timestamp getTimestamp(int columnId) throws NdbApiException;

    /**
     *
     * @param columnName
     * @return a Timestamp object
     * @throws NdbApiException if a value of this type cannot be retrieved as the next entry in the NdbResultSet
     */
    public abstract Timestamp getTimestamp(String columnName) throws NdbApiException;

    /**
     *
     * @param columnId column number (starting from '0')
     * @return a byte array that is allocated in JNI. The caller owns the byte array,
     * so does not need to make a copy of it.
     * @throws NdbApiException if a value of this type cannot be retrieved as the next entry in the NdbResultSet
     */
    public abstract byte[] getBytes(int columnId) throws NdbApiException;

    /**
     * Returns a byte array for a specified column if it is in the result set.
     * @param columnName name of the column in the schema
     * @return a byte array that is allocated in JNI. The caller owns the byte array,
     * so does not need to make a copy of it.
     * @throws NdbApiException if a value of this type cannot be retrieved as the next entry in the NdbResultSet
     */
    public abstract byte[] getBytes(String columnName) throws NdbApiException;

    /**
     *
     * @param columnId column number (starting from '0')
     * @return a String object
     * @throws NdbApiException if a value of this type cannot be retrieved as the next entry in the NdbResultSet
     */
    public abstract String getString(int columnId) throws NdbApiException;

    /**
     *
     * @param columnName
     * @return a String object
     * @throws NdbApiException if a value of this type cannot be retrieved as the next entry in the NdbResultSet
     */
    public abstract String getString(String columnName) throws NdbApiException;

    /**
     * Call the next() method to get the next entry in the NdbResultSet.
     * @return true if there is another value in the NdbResultSet, false if there are no more entries in the NdbResultSet.
     * @throws NdbApiException if a problem occurred when attempting to get the next row.
     */
    public abstract boolean next() throws NdbApiException;
    public abstract boolean next(boolean fetchAllowed) throws NdbApiException;
    
    /**
     * Deletes the next row in the NdbResultSet.
     * @throws NdbApiException if a problem occurred when attempting to delete the row.
     */
    public abstract void deleteRow() throws NdbApiException;

    /**
     * Updates the next row in the NdbResultSet.
     * @return a NdbOperation object
     * @throws NdbApiException  if a problem occurred when attempting to update the row.
     */
    public abstract NdbOperation getUpdateOperation() throws NdbApiException;
    public NdbOperation getUpdateOperation(NdbTransaction trans) throws NdbApiException; 
    /**
     * Reports whether the last column read had a value of SQL NULL.
     * Note that you must first call one of the getter methods (e.g., getInt(..)) on a column to
     * on the NdbResultSet (e.g., rs.getString() ) try to read its value and
     * then call the method wasNull to see if the value read was SQL NULL
     * @return true if the last column read was null (SQL NULL), false if it was not null  (SQL NULL).
     * @throws NdbApiException if a problem occurred, such as not having called get<TYPE>() on the NdbResultSet object before calling the wasNull() method.
     */
    public abstract boolean wasNull() throws NdbApiException;

	/**
	 * @see com.mysql.cluster.ndbj.NdbResultSet#getBlob(int)
	 */
	public NdbBlob getBlob(int columnId) throws NdbApiException;

	/**
	 * @see com.mysql.cluster.ndbj.NdbResultSet#getBlob(java.lang.String)
	 */
	public NdbBlob getBlob(String columnName) throws NdbApiException;

}
