package com.mysql.cluster.ndbj;

/**
 * The NdbBaseScan interface defines common methods providded by
 * both NdbScanOperation and NdbIndexScanOperation objects.
 * <p>A scan operation reads many rows in a table; whereas a primary key operation
 * (provided by NdbOperation objects) read a single row in a table.
 * <p>Scans are executed in parallel over the nodes in a cluster,
 * and is therefore an expensive operation to perform. Use sparingly.
 * <br>Call the close method on the scan object to finish a scan when it is no longer needed.
 * <p>An IllegalStateException is thrown if you attempt to invoke methods on concrete instances of this type
 * after the close method has been called on those instances.
 *
 * <br><br>The following is an example of a range scan on an Index
 * in mysql (using the mysql client utility):
 * <code>
 * <br><br>mysql_client>create table t_idx(id int, name varchar(64), index name_idx(name)) engine=ndb;
 * <br>mysql_client>insert into t_idx values (1,'abba'), (2,'mouse'),(3,'zebra');
 * <br>mysql_client>select * from t_idx where name > 'm';
 * </code>
 * <br><br>To see how the name_idx index is used, use the explain function in mysql:
 * <code>
 * <br>mysql_client>explain select * from t_idx where name > 'm';
 * </code>
 *
 * @see com.mysql.cluster.ndbj.NdbScanOperation
 * @see com.mysql.cluster.ndbj.NdbOperation
 * @see com.mysql.cluster.ndbj.NdbBase
 * @see com.mysql.cluster.ndbj.NdbTransaction
 * @see com.mysql.cluster.ndbj.Ndb
 * @see com.mysql.cluster.ndbj.NdbClusterConnectionImpl
 */
public interface NdbScanOperation extends NdbOperation {

    /**
     * 
     * A ScanFlag can be supplied to a readTuples operation to indicate
     * how the data should be ordered before being returned to the client.
     * 
     * @author jdowling
     */
	public enum ScanFlag {
        /**
         * SF_TupScan is 0x10000
         * <p>
         * In C++ NDB-API, these Scan Types are defined in NdbScanOperation.hpp
         */
		TUPLE_SCAN(1 << 16),
        /**
         * SF_OrderBy is 0x1000000
         */
		ORDER_BY(1 << 24),
        /**
         * SF_Descending is 0x2000000
         */
		DESCENDING(2 << 24),
        /**
         * SF_ReadRangeNo is 0x4000000
         */
		READ_RANGE_NUM(4 << 24),
        /**
         * TODO: SF_KeyInfo indicate that index information is ??.
         */
		KEY_INFO(1),
        
        /**
         * No flag is passed by the client. 
         */
        NO_FLAG(0);

		public int type;
		private ScanFlag(int type) {
			this.type = type;
		}
		public int swigValue() {
			return this.type;
		}

	}



	/**
	 * This method is used by the NdbResultSet to fetch the next tuple in a scan transaction.
	 * Following each call to nextResult(), the NdbResultSet objects
	 * populated using NdbOperation.getValue() are updated
	 * with values from the scanned tuple.
	 *
	 * @param fetchAllowed
	 * @param forceSend
	 * @return
	 */
	int nextResult(boolean fetchAllowed, boolean forceSend) throws NdbApiException ;
	int nextResult(boolean fetchAllowed) throws NdbApiException ;
	int nextResult() throws NdbApiException ;

	/**
	 * Calling this method closes a scan.
	 *
	 * @param forceSend when this parameter is set to true, it forces transactions to be sent to the kernel.
	 * @param releaseOp set to true in order to release the operation and free its resources.
	 */
	void close(boolean forceSend, boolean releaseOp);

	/**
	 * Calling this method closes a scan.
	 *
	 * @param forceSend when this parameter is set to true, it forces transactions to be sent to the kernel.
	 */
	void close(boolean forceSend);

	/**
	 * Calling this method closes a scan.
	 * <p>It does not force its transactions to be sent to the kernel or release the operation and free its resources.
	 * @see close(boolean forceSend, boolean releaseOp)
	 */
	void close();

	/**
	 * Used by NdbResultSet to updates the current tuple in the scan
	 * @return
	 * @throws NdbApiException if there is an internal problem in the cluster.
	 */
	NdbOperation updateCurrentTuple() throws NdbApiException;

	/**
	 * This method is used to update the current tuple using the transaction object
	 * passed as a parameter.
	 * @param updateTrans
	 * @return NdbOperation object or null if no operation is updated
	 * @throws NdbApiException if there is an internal problem in the cluster.
	 */
	NdbOperation updateCurrentTuple(NdbTransactionImpl updateTrans)
			throws NdbApiException;

	/**
	 * This method is used to delete the current tuple using the current transaction.
	 *
	 * @throws NdbApiException if there is an internal problem in the cluster.
	 */
	void deleteCurrentTuple() throws NdbApiException;

	/**
	 * This method is used to delete the current tuple using a transaction
	 * passed as a parameter.
	 * @param deleteTrans the transaction object used to perform the deletion.
	 * @throws NdbApiException if there is an internal problem in the cluster
	 */
	void deleteCurrentTuple(NdbTransactionImpl deleteTrans)
			throws NdbApiException;

	/**
	 *  This method locks the current tuple using the current transaction.
	 * @throws NdbApiException if there is an internal problem in the cluster
	 */
	NdbOperation lockCurrentTuple()
	throws NdbApiException;

	/**
	 *  This method locks the current tuple using the transaction
	 *  object passed as a parameter.
	 * @param trans the transaction that should perform the lock.
	 * @throws NdbApiException if there is an internal problem in the cluster
	 */
	NdbOperation lockCurrentTuple(NdbTransactionImpl trans)
	throws NdbApiException;

	/**
	 * Restart scan with exactly the same getValues and search conditions
	 * @param boolean forceSend if true, do not buffer request, flush immediately to
	 * cluster @return -1 on error, 0 on success
	 * @throws NdbApiException if there is an internal problem in the cluster
	 */
	void restart(boolean forceSend) throws NdbApiException;

	/**
	 * Restart scan with exactly the same getValues and search conditions
	 * @throws NdbApiException if there is an internal problem in the cluster
	 */
	void restart() throws NdbApiException;

	/**
	 * Creates a new NdbScanFilter that can be applied to
	 * filter the results returned from the scan operation.
	 * Note: call getValue() methods on the NdbScanOperation object
	 * before you call begin on a NdbScanFilter object.
	 *
	 * @return
	 * @throws NdbApiException
	 */
	NdbScanFilter getNdbScanFilter() throws NdbApiException;


}
