package com.mysql.cluster.ndbj;

import java.math.BigInteger;
import javax.transaction.Transaction;

/**
 * A transaction (represented by an NdbTransaction object) belongs to an Ndb
 * object and is created using Ndb.startTransaction(). A transaction consists of
 * a list of operations (represented by NdbOperation, NdbScanOperation,
 * NdbIndexOperation, and NdbIndexScanOperation objects). Each operation access
 * exactly one table.
 * 
 * After getting the NdbTransaction object, the first step is to get an
 * operation given the table name using one of the methods #getNdbOperation(),
 * #getNdbScanOperation(), #getNdbIndexOperation(), or
 * #getNdbIndexScanOperation(). Then the operation is defined. Several
 * operations can be defined on the same NdbTransaction object, they will in
 * that case be executed in parallell. When all operations are defined, the
 * #execute() method sends them to the NDB kernel for execution.
 * 
 * The #execute() method returns when the NDB kernel has completed execution of
 * all operations defined before the call to #execute(). All allocated
 * operations should be properly defined before calling #execute().
 * 
 * A call to execute() uses one out of three types of execution: -#
 * ExecType.NoCommit Executes operations without committing them. -#
 * ExecType.Commit Executes remaining operation and commits the complete
 * transaction -# ExecType.Rollback Rollbacks the entire transaction.
 * 
 * execute() is equipped with an extra error handling parameter. There are two
 * alternatives: -# AbortOption.AbortOnError (default). The transaction is
 * aborted if there are any error during the execution -#
 * AbortOption.AO_IgnoreError Continue execution of transaction even if
 * operation fails
 * 
 * <br>
 * Sample Code:<br>
 * <code>
 *      NdbClusterConnection conn=null;<br>
 *      Ndb ndb=null;<br>
 *      NdbTransaction trans=null;<br>
 *      try {<br>
 *          conn = NdbFactory.createNdbClusterConnection("ndb-connectstring");<br>
 *          ndb = conn.createNdb("database",12);// Max num of concurrent transactions is set to 12 <br>
 *          trans = ndb.startTransaction();<br>
 *          // use the transaction
 *      }<br>
 *      catch (NdbApiException e) {<br>
 *          // Handle Error
 *      }<br>
 *      finally {<br>
 *          // You must close the ndb object to free up native resources.<br>
 *          if  (trans != null)<br>
 *              trans.close();<br>
 *          if  (ndb != null)<br>
 *              ndb.close();<br>
 *          if (conn != null)<br>
 *              conn.close();<br>
 *      }<br>
 *</code>
 * 
 * @see com.mysql.cluster.ndbj.Ndb
 * @see com.mysql.cluster.ndbj.NdbClusterConnection
 * @see com.mysql.cluster.ndbj.NdbOperation
 * @see com.mysql.cluster.ndbj.NdbIndexOperation
 * @see com.mysql.cluster.ndbj.NdbScanOperation
 * @see com.mysql.cluster.ndbj.NdbIndexScanOperation
 * 
 */
public interface NdbTransaction extends Transaction, NdbOperationFactory {

	/**
	 * This type is used to determine the state of the transaction.
	 */
	public enum ExecutionState {
		/**
		 * The transaction has been started.
		 */
		STARTED,
		/**
		 * The transaction has been executed but not committed.
		 */
		EXECUTED_NOT_COMMITTED,
		/**
		 * The transaction has been executed and committed.
		 */
		EXECUTED_COMMITTED,
		/**
		 * The transaction has been rolled back.
		 */
		EXECUTED_ROLLED_BACK,
		/**
		 * The transaction has been closed.
		 */
		CLOSED,
		/**
		 * The transaction is in an invalid state.
		 */
		ERROR;

		private ExecutionState() {
		}
	};

	/*
	 * The transaction stores a list of Blob objects for the transactions. 
	 * Used internally in NDB/J.
	 * @param blob object
	 
	 abstract void addNdbBlob(NdbBlob blob);*/
	/**
	 * This method executes the transaction and all the operations that have
	 * been added it. It either succeeds or fails.
	 * 
	 * The execute method returns when the NDB kernel has completed execution of
	 * all operations defined before the call to execute. All allocated
	 * operations should be properly defined before calling execute. A call to
	 * the execute method uses one out of three types of execution: Commit,
	 * NoCommit, or Rollback. If an operation in the transaction fails, it is
	 * aborted if abortOption is set to AbortOption.AbortOnError , and the
	 * transaction continues excution after the failed operation if abortOption
	 * is AbortOption.AO_ignoreError.
	 * <p>
	 * If forceSend is set to 0, the transaction is placed in a send buffer for
	 * up to a maximum of 10 milliseconds, after which it is flushed to the
	 * cluster. This can help improve system performance in high load, through
	 * increased batching of transactions. If forceSend is set to 1, the
	 * transaction is flushed immediately to the cluster. This improves
	 * transaction performance, but may degrade system scalability (throughput
	 * of the cluster in terms of number of transactions that can be executed
	 * per time unit).
	 * 
	 * @param execType
	 *            specifies whether the transaction should be committed,
	 *            not-committed or rolled-back. <br>
	 *            ExecType.NoCommit : executes operations without committing
	 *            them. <br>
	 *            ExecType.Commit : executes remaining operation and commits the
	 *            complete transaction <br>
	 *            ExecType.Rollback : rollbacks the entire transaction.
	 * @param abortOption
	 *            specifies whether the transaction should be aborted or not if
	 *            one of its operations fails. <br>
	 *            AbortOption.AbortOnError : (default) The transaction is
	 *            aborted if there are any error during the execution <br>
	 *            AbortOption.AO_IgnoreError : Continue execution of transaction
	 *            even if operation fails
	 * @param force
	 *            <br>
	 *            true : put the transaction in the send buffer <br>
	 *            false : flush the send buffer so that the transaction executes
	 *            immediately (improves transaction performance, may degrade
	 *            system performance)
	 * @throws NdbApiException
	 *             <br>
	 *             base exception. Catch this if you don't care about retrying
	 *             transactions that failed because of temporary problems in the
	 *             cluster.
	 * @throws NdbApiTemporaryException
	 *             <br>
	 *             transaction failed. If this exception is caught, retry the
	 *             transaction. The transaction failed due a temporary problem
	 *             (e.g., a single node failed), and retrying the transaction
	 *             may cause it to succeed.
	 * @throws NdbApiPermanentException
	 *             <br>
	 *             transaction failed. if this exception is caught, the
	 *             transaction should not be retried, as it failed due to some
	 *             serious error
	 * @throws NdbApiUserAndPermanentException
	 *             <br>
	 *             transaction failed. if this exception is caught, the
	 *             transaction should not be retried, as it failed due to some
	 *             serious (possibly user) error (such as deleting a row that
	 *             doesn't exist).
	 */
	int execute(ExecType execType, AbortOption abortOption, boolean force)
			throws NdbApiException;

	/**
	 * This method executes the transaction and all the operations that have
	 * been added it. It either succeeds or fails.
	 * 
	 * The execute method returns when the NDB kernel has completed execution of
	 * all operations defined before the call to execute. All allocated
	 * operations should be properly defined before calling execute. A call to
	 * the execute method uses one out of three types of execution: Commit,
	 * NoCommit, or Rollback. If an operation in the transaction fails, it is
	 * aborted if abortOption is set to AbortOption.AbortOnError , and the
	 * transaction continues excution after the failed operation if abortOption
	 * is AbortOption.AO_ignoreError.
	 * <p>
	 * If forceSend is set to 0, the transaction is placed in a send buffer for
	 * up to a maximum of 10 milliseconds, after which it is flushed to the
	 * cluster. This can help improve system performance in high load, through
	 * increased batching of transactions. If forceSend is set to 1, the
	 * transaction is flushed immediately to the cluster. This improves
	 * transaction performance, but may degrade system scalability (throughput
	 * of the cluster in terms of number of transactions that can be executed
	 * per time unit).
	 * 
	 * @param execType
	 *            specifies whether the transaction should be committed,
	 *            not-committed or rolled-back. <br>
	 *            ExecType.NoCommit : executes operations without committing
	 *            them. <br>
	 *            ExecType.Commit : executes remaining operation and commits the
	 *            complete transaction <br>
	 *            ExecType.Rollback : rollbacks the entire transaction.
	 * @param abortOption
	 *            specifies whether the transaction should be aborted or not if
	 *            one of its operations fails. <br>
	 *            AbortOption.AbortOnError : (default) The transaction is
	 *            aborted if there are any error during the execution <br>
	 *            AbortOption.AO_IgnoreError : Continue execution of transaction
	 *            even if operation fails
	 * @throws NdbApiException
	 *             <br>
	 *             base exception. Catch this if you don't care about retrying
	 *             transactions that failed because of temporary problems in the
	 *             cluster.
	 * @throws NdbApiTemporaryException
	 *             <br>
	 *             transaction failed. If this exception is caught, retry the
	 *             transaction. The transaction failed due a temporary problem
	 *             (e.g., a single node failed), and retrying the transaction
	 *             may cause it to succeed.
	 * @throws NdbApiPermanentException
	 *             <br>
	 *             transaction failed. if this exception is caught, the
	 *             transaction should not be retried, as it failed due to some
	 *             serious error
	 * @throws NdbApiUserAndPermanentException
	 *             <br>
	 *             transaction failed. if this exception is caught, the
	 *             transaction should not be retried, as it failed due to some
	 *             serious (possibly user) error (such as deleting a row that
	 *             doesn't exist).
	 */
	int execute(ExecType execType, AbortOption abortOption)
			throws NdbApiException;

	/**
	 * This method executes the transaction and all the operations that have
	 * been added it. It either succeeds or fails.
	 * 
	 * The execute method returns when the NDB kernel has completed execution of
	 * all operations defined before the call to execute. All allocated
	 * operations should be properly defined before calling execute. A call to
	 * the execute method uses one out of three types of execution: Commit,
	 * NoCommit, or Rollback. If an operation in the transaction fails, it is
	 * aborted if abortOption is set to AbortOption.AbortOnError , and the
	 * transaction continues excution after the failed operation if abortOption
	 * is AbortOption.AO_ignoreError.
	 * <p>
	 * If forceSend is set to 0, the transaction is placed in a send buffer for
	 * up to a maximum of 10 milliseconds, after which it is flushed to the
	 * cluster. This can help improve system performance in high load, through
	 * increased batching of transactions. If forceSend is set to 1, the
	 * transaction is flushed immediately to the cluster. This improves
	 * transaction performance, but may degrade system scalability (throughput
	 * of the cluster in terms of number of transactions that can be executed
	 * per time unit).
	 * 
	 * @param execType
	 *            specifies whether the transaction should be committed,
	 *            not-committed or rolled-back. <br>
	 *            ExecType.NoCommit : executes operations without committing
	 *            them. <br>
	 *            ExecType.Commit : executes remaining operation and commits the
	 *            complete transaction <br>
	 *            ExecType.Rollback : rollbacks the entire transaction.
	 * @throws NdbApiException
	 *             <br>
	 *             base exception. Catch this if you don't care about retrying
	 *             transactions that failed because of temporary problems in the
	 *             cluster.
	 * @throws NdbApiTemporaryException
	 *             <br>
	 *             transaction failed. If this exception is caught, retry the
	 *             transaction. The transaction failed due a temporary problem
	 *             (e.g., a single node failed), and retrying the transaction
	 *             may cause it to succeed.
	 * @throws NdbApiPermanentException
	 *             <br>
	 *             transaction failed. if this exception is caught, the
	 *             transaction should not be retried, as it failed due to some
	 *             serious error
	 * @throws NdbApiUserAndPermanentException
	 *             <br>
	 *             transaction failed. if this exception is caught, the
	 *             transaction should not be retried, as it failed due to some
	 *             serious (possibly user) error (such as deleting a row that
	 *             doesn't exist).
	 */
	int execute(ExecType execType) throws NdbApiException;

	/**
	 * This method should only be used after the transaction has been executed,
	 * but before the transaction has been closed.
	 * 
	 * Transactions consist of groups of operations. When a transaction has been
	 * executed, you can call this method to get the next completed operation in
	 * a transaction after the supplied operation object. If null is specified
	 * as a parameter, then the first completed operation object is returned.
	 * 
	 * @param anOperation
	 *            an operation object in the transaction. If op is null, the
	 *            first completed operation in the transaction is returned.
	 * @return the next completed operation object in the transaction.
	 * @throws NdbApiException
	 *             if there is an error in the JNI library
	 * @throws IllegalStateException
	 *             if the close method has already been called on the object.
	 */
	NdbOperation getNextCompletedOperation(NdbOperationImpl anOperation)
			throws NdbApiException;

	/**
	 * closes the NdbTransaction object. The object can no longer be used after
	 * the close() method has been invoked. You have to create a new
	 * NdbTransaction object using a Ndb.startTransaction() method.
	 */
	void close();

	/**
	 * checks if the transaction object is closed
	 * @return true if closed, false if not closed
	 */
	boolean isClosed();

	/**
	 * Returns the number of rows in the supplied table name.
	 * 
	 * @param tableName
	 *            name of Schema
	 * @return number of rows in table
	 * @throws NdbApiException
	 * @throws IllegalStateException
	 *             if the close method has already been called on the object.
	 */
	BigInteger selectCount(String tableName) throws NdbApiException;

	/**
	 * restart the transaction. This should not be called after the transaction
	 * has been closed.
	 * 
	 * @return
	 * @throws NdbApiException
	 * @throws IllegalStateException
	 *             if the close method has already been called on the object.
	 */
	int restart() throws NdbApiException;

	/**
	 * Prepare an asynchronous transaction.
	 * 
	 * 
	 * @param execType
	 *            {@link #execute(ExecType)}
	 * @param callback
	 *            A callback object. This handleCallback method of this object
	 *            gets called when the transaction has been executed.
	 * @param abortOption
	 *            see {@link #execute(ExecType, AbortOption)}
	 */
	void executeAsynchPrepare(ExecType execType, BaseCallback callback,
			AbortOption abortOption);

	/**
	 * Prepare an asynchronous transaction.
	 * 
	 * 
	 * @param execType
	 *            {@link #execute(ExecType)}
	 * @param callback
	 *            A callback object. The handleCallback method of this object
	 *            gets called when the transaction has been executed.
	 */
	void executeAsynchPrepare(ExecType execType, BaseCallback callback);

	int executeNoCommit(AbortOption abortOption) throws NdbApiException,
			NdbApiTemporaryException, NdbApiPermanentException;

	int executeNoCommit() throws NdbApiException,
			NdbApiTemporaryException, NdbApiPermanentException;

	int executeCommit(AbortOption abortOption) throws NdbApiException,
			NdbApiTemporaryException, NdbApiPermanentException;

	int executeCommit() throws NdbApiException,
			NdbApiTemporaryException, NdbApiPermanentException;

	int executeRollback(AbortOption abortOption) throws NdbApiException,
			NdbApiTemporaryException, NdbApiPermanentException;

	int executeRollback() throws NdbApiException,
			NdbApiTemporaryException, NdbApiPermanentException;

}
