package testsuite.ndbj;

import testsuite.BaseNdbjTestCase;

import com.mysql.cluster.ndbj.NdbApiException;
import com.mysql.cluster.ndbj.NdbOperation;
import com.mysql.cluster.ndbj.NdbScanFilter;
import com.mysql.cluster.ndbj.NdbScanOperation;
import com.mysql.cluster.ndbj.NdbTransaction;
import com.mysql.cluster.ndbj.NdbOperation.LockMode;
import com.mysql.cluster.ndbj.NdbScanFilter.BinaryCondition;
import com.mysql.cluster.ndbj.NdbScanFilter.Group;
import com.mysql.cluster.ndbj.AbortOption;
import com.mysql.cluster.ndbj.ExecType;

import java.sql.SQLException;


public class NdbScanFilterTest extends BaseNdbjTestCase
{
    private final static int NUM_ENTRIES=10000;
    protected final static String tablename="t_scan_filter";
    protected String col1="id";
    protected String col2="utf8LongVarChar";
    protected String col3="reference";
    protected String col4="long_num";
    protected String col5="latin1VarChar";
    protected String col6="char";
    protected String col7="LongVarBinary";

    protected int strSize=100;

    public static void main(String[] args)
    {
        junit.textui.TestRunner.run(NdbOperationTest.class);
    }

    public NdbScanFilterTest(String arg0)
    {
        super(arg0);
    }

    @Override
    public void setUp() throws Exception
    {
        super.setUp();
        createTable(tablename,
                "(`"+col1+"` INT PRIMARY KEY, `"+col2+"` VARCHAR(256) CHARSET utf8, `"
                +col3+"` INT, `"+col4+"` BIGINT, `"+col5+"` VARCHAR("+strSize+"), `" +col6
                + "` CHAR(20), `" + col7 + "` VARBINARY(256)"
                + ") ENGINE=ndb;"); //CHARSET utf8
    }

    private static interface ScanDetails
    {
        public void defineFiltering(NdbScanOperation scan) throws Exception;
    }

    private static ScanDetails noFilter=new ScanDetails() {
        public void defineFiltering(NdbScanOperation scan)
        {

        }
    };

    private void generalScan(String testName, ScanDetails sd, int resultsExpected, int iterations) throws Exception
    {
        long minTime=100000;
        long maxTime=0;
        long total=0;

        System.out.println("Beginning test of : "+testName+"...");

        int tupleCount=0;
        NdbTransaction scanTrans = null;

        try {
	        for (int iter=0; iter<iterations; iter++)
	        {
	            scanTrans = ndb.startTransaction();

	            trans.setOperationLockMode(LockMode.LM_CommittedRead);
	            
	            NdbScanOperation scan = scanTrans.getSelectScanOperation(tablename);

	            sd.defineFiltering(scan);

	            long startTime=System.currentTimeMillis();
	            rs = scan.resultData();

	            scanTrans.execute(ExecType.NoCommit, AbortOption.AbortOnError, true);


	            tupleCount=0;
	            while (rs.next())
	            {
	                tupleCount++;
	            }

	            long duration=System.currentTimeMillis()-startTime;
	            if (duration > maxTime)
	            {
	                maxTime=duration;
	            }
	            if (duration < minTime)
	            {
	                minTime=duration;
	            }
	            total+=duration;
	            assertEquals(resultsExpected, tupleCount);
	            scanTrans.execute(ExecType.Commit, AbortOption.AbortOnError, true);

	            scan.close(true, true);
	       		scanTrans.close();
	       	 
	        }
        }
        catch (SQLException e) {
        	//rethrow only
        	throw e;
        }
        finally {
        	if (scanTrans != null) {
        		scanTrans.close();
        	}
        }
        System.out.println("Iterations : "+iterations+".  Results per iteration="+tupleCount
                +".  Total time="+total
                +"  Avg time="+(total/iterations)
                +".  Max time="+maxTime+".  Min time="+minTime+". (Times in millis)");
    }

    private class colNIntMatchFilter implements ScanDetails
    {
        int value;
        int column;

        public colNIntMatchFilter(int _column, int _value)
        {
            value=_value;
            column=_column;
        }

        public void defineFiltering(NdbScanOperation scan) throws Exception
        {
            NdbScanFilter sf=scan.getNdbScanFilter();

            sf.begin(Group.AND);
            sf.eq(column, value);
            sf.end();
        }
    }

    private class colNLongMatchFilter implements ScanDetails
    {
        long value;
        int column;

        public colNLongMatchFilter(int _column, long _value)
        {
            value=_value;
            column=_column;
        }

        public void defineFiltering(NdbScanOperation scan) throws Exception
        {
             NdbScanFilter sf=scan.getNdbScanFilter();

             sf.begin(Group.AND);
             sf.eq(column,value);
             sf.end();
        }
    }

    private class colNStringMatchFilter implements ScanDetails
    {
        String value;
        int column;

        public colNStringMatchFilter(int _column, String _value)
        {
            value=_value;
            column=_column;
        }

        public void defineFiltering(NdbScanOperation scan) throws NdbApiException
        {
            NdbScanFilter sf=scan.getNdbScanFilter();

            sf.eq(column, value);
            
          
        }
    }

    private class colNLikeMatchFilter implements ScanDetails
    {
        String value;
        int column;

        public colNLikeMatchFilter(int _column, String _value)
        {
            value=_value;
            column=_column;
        }

        public void defineFiltering(NdbScanOperation scan) throws Exception
        {
            NdbScanFilter sf=scan.getNdbScanFilter();

            sf.begin(Group.AND);
            // TODO: Fix naming issue
            sf.cmpString(BinaryCondition.COND_LIKE, column, value);
            sf.end();
       
        }
    }


    public void testScan() throws Exception
    {


        // First, datafill the table with appropriate data

        try {
	        trans=ndb.startTransaction();


	        for (int i=0; i<NUM_ENTRIES-1; i++)
	        {

	            NdbOperation op = trans.getInsertOperation(tablename);

	            op.setInt(col1, i);
	            op.setString(col2, "the same");
	            op.setInt(col3, 12);
	            op.setLong(col4, 12);
	            op.setString(col5, Integer.toString(12));
	            op.setString(col6, Integer.toString(12));
	            op.setString(col7, Integer.toString(12));
	        }
	        trans.execute(ExecType.Commit, AbortOption.AbortOnError, true);
        }
        catch (NdbApiException e) {
        	System.out.println(e.toString());

        }
        finally {
        	if (trans != null) {
        		trans.close();
        	}
        }

        try {

	        // Add a non-matching entry...
	        trans=ndb.startTransaction();

	        NdbOperation op = trans.getInsertOperation(tablename);

	        op.setInt(col1, NUM_ENTRIES);
	        op.setString(col2, "different");
	        op.setInt(col3, 13);
	        op.setLong(col4, 13);
	        op.setString(col5, Integer.toString(13));
	        op.setString(col6, Integer.toString(13));
	        op.setString(col7, Integer.toString(13));

	        trans.execute(ExecType.Commit, AbortOption.AbortOnError, true);
        }
        catch (NdbApiException e) {
        	System.out.println(e.toString());

        }
        finally {
        	trans.close();
        }

        final int ITERATIONS=100;

        generalScan("Open scan - no filter", noFilter, NUM_ENTRIES, ITERATIONS);

        generalScan("Match most entries on int", new colNIntMatchFilter(3, 12), NUM_ENTRIES-1, ITERATIONS);
        generalScan("Match one entry on int", new colNIntMatchFilter(3, 13), 1, ITERATIONS);
        generalScan("Match zero entries on int", new colNIntMatchFilter(3, 14), 0, ITERATIONS);

        generalScan("Match most entries on int", new colNLongMatchFilter(4, 12), NUM_ENTRIES-1, ITERATIONS);
        generalScan("Match one entry on int", new colNLongMatchFilter(4, 13), 1, ITERATIONS);
        generalScan("Match zero entries on int", new colNLongMatchFilter(4, 14), 0, ITERATIONS);

        generalScan("Match most entries on string (EQUALS)", new colNStringMatchFilter(5, "12"), NUM_ENTRIES-1, ITERATIONS);
        generalScan("Match one entry on string (EQUALS)", new colNStringMatchFilter(5, "13"), 1, ITERATIONS);
        generalScan("Match zero entries on a string (EQUALS)", new colNStringMatchFilter(5, "14"), 0, ITERATIONS);

        generalScan("Match most entries on string compare (LIKE)", new colNLikeMatchFilter(5, "12"), NUM_ENTRIES-1, ITERATIONS);
        generalScan("Match one entry on string (LIKE)", new colNLikeMatchFilter(5, "13"), 1, ITERATIONS);
        generalScan("Match zero entries on a string (LIKE)", new colNLikeMatchFilter(5, "14"), 0, ITERATIONS);

        generalScan("Match most entries on char (EQUALS)", new colNStringMatchFilter(6, "12"), NUM_ENTRIES-1, ITERATIONS);
        generalScan("Match one entry on char (EQUALS)", new colNStringMatchFilter(6, "13"), 1, ITERATIONS);
        generalScan("Match zero entries on a char (EQUALS)", new colNStringMatchFilter(6, "14"), 0, ITERATIONS);

        generalScan("Match most entries on char compare (LIKE)", new colNLikeMatchFilter(6, "12"), NUM_ENTRIES-1, ITERATIONS);
        generalScan("Match one entry on char (LIKE)", new colNLikeMatchFilter(6, "13"), 1, ITERATIONS);
        generalScan("Match zero entries on a char (LIKE)", new colNLikeMatchFilter(6, "14"), 0, ITERATIONS);

        generalScan("Match most entries on varbinary (EQUALS)", new colNStringMatchFilter(7, "12"), NUM_ENTRIES-1, ITERATIONS);
        generalScan("Match one entry on varbinary (EQUALS)", new colNStringMatchFilter(7, "13"), 1, ITERATIONS);
        generalScan("Match zero entries on a varbinary (EQUALS)", new colNStringMatchFilter(7, "14"), 0, ITERATIONS);

        generalScan("Match most entries on VARBINARY compare (LIKE)", new colNLikeMatchFilter(6, "12"), NUM_ENTRIES-1, ITERATIONS);
        generalScan("Match one entry on VARBINARY (LIKE)", new colNLikeMatchFilter(6, "13"), 1, ITERATIONS);
        generalScan("Match zero entries on a VARBINARY (LIKE)", new colNLikeMatchFilter(6, "14"), 0, ITERATIONS);

    }
}

