package testsuite.ndbj;

import java.io.UnsupportedEncodingException;
import java.sql.Timestamp;

import java.sql.SQLException;

import testsuite.BaseNdbjTestCase;

import com.mysql.cluster.ndbj.NdbApiException;
import com.mysql.cluster.ndbj.NdbBlob;
import com.mysql.cluster.ndbj.NdbIndexScanOperation;
import com.mysql.cluster.ndbj.NdbOperation;
import com.mysql.cluster.ndbj.NdbResultSet;
import com.mysql.cluster.ndbj.NdbScanOperation;
import com.mysql.cluster.ndbj.NdbOperation.LockMode;

public class NdbTest extends BaseNdbjTestCase {

    static final String PRIMARYKEYINDEXNAME = "PRIMARY";

    static final String TABLE1 = "table1";
    static final String TABLE2 = "table2";
    static final String TABLE3 = "table3";
    static final String key1   = "k1";
    static final String key2   = "k2";
    static final String data1  = "d1";
    static final String data2  = "d2";
    static final String data3  = "d3";
    static final String data4  = "d4";
    static final String data5  = "d5";
    static final String data6  = "d6";
    static final String data7  = "d7";

    static final String rowTable1 = "rob";
    static final String rowTable2 = "james";
    static final String rowTable3 = "maeve";

    static final int TEST_BLOB_SIZE = 8*1024;

    public static void main(String[] args) {
        junit.textui.TestRunner.run(NdbTest.class);
    }

    public NdbTest(String arg0) {
        super(arg0);
    }

    @Override
    public void setUp() throws Exception {
        super.setUp();
        createTable(TABLE1,
                    "( k1 VARCHAR(30) NOT NULL, d1 VARCHAR(20), " +
                    " d2 INTEGER, primary key (k1) ) engine=ndbcluster;");

        createTable(TABLE2,
                    "( k1 VARCHAR(256) NOT NULL, k2 INTEGER NOT NULL, " +
                    " d1 VARCHAR(256)," +
                    " d2 INTEGER, primary key (k1, k2) ) engine=ndbcluster;");

        createTable(TABLE3,
                    "( k1 VARCHAR(256) NOT NULL, d1 TIMESTAMP, d2 CHAR(1)," +
                    " d3 VARBINARY(100), d4 BLOB, d5 TEXT, d6 BIGINT," +
                    " d7 BLOB, PRIMARY KEY (k1)) engine=ndbcluster;");

    }

	private static void printTable1( NdbResultSet rs ) throws NdbApiException
	{
		System.out.println("    Table1 results");

		while( rs.next() )
		{
			String  k1 = rs.getString(key1);
			String  d1 = rs.getString(data1);
			Integer d2 = new Integer(rs.getInt(data2));

			System.out.printf("      Row: %-10s %-20s %10d \n", k1, d1, d2 );
		}

	}

	private static void printTable2( NdbResultSet rs ) throws NdbApiException
	{
		System.out.println("    Table2 results");

		while( rs.next() )
		{
			String  k1 = rs.getString(key1);
			Integer k2 = new Integer(rs.getInt(key2));
			String  d1 = rs.getString(data1);
			Integer d2 = new Integer(rs.getInt(data2));

			System.out.printf("      Row: %-10s %10s %-20s %10d \n", k1, k2, d1, d2 );
		}
	}


    private static String convertToHexString( byte[] bytes )
    {
        StringBuilder builder = new StringBuilder();

        for( byte b : bytes )
        {
            builder.append( Integer.toHexString(b) );
            builder.append(",");
        }

        return builder.toString();
    }



    private static String convertToHexString( byte[] bytes, int length )
    {
        StringBuilder builder = new StringBuilder();

        if( length > bytes.length )
        {
            builder.append( "Invalid length!\n");
            length = bytes.length;
        }

        for( int idx=0; idx<length; idx++ )
        {
            builder.append( Integer.toHexString(bytes[idx]) );
            builder.append(",");
        }

        return builder.toString();
    }

	/*
	 * Test method for 'com.mysql.ndbapi.Ndb.destroy()'
	 */

	public void testNdb() throws Exception
	{
		System.out.println("\nTest: insertTable1");

		try {
			trans = ndb.startTransaction();

			NdbOperation op = trans.getInsertOperation(TABLE1);

			op.setString( key1,  rowTable1);
			op.setString( data1, null );
			op.setInt(    data2, 42 );

			trans.executeNoCommit();


			trans.executeCommit();
		}
		finally {
			if (trans != null)
				trans.close();
		}
		
		
        System.out.println("\nTest: insertTable1 Expecting Error");

        NdbOperation op = null;

        try
        {
            trans = ndb.startTransaction();

            op = trans.getInsertOperation(TABLE1);

            op.setString( key1,  rowTable1 );
            op.setString( data1, "mynewvalue");
            op.setInt(    data2, 42 );

            trans.executeNoCommit();
            trans.executeRollback();

            fail("Exception should be thrown");
        }
        catch(NdbApiException e)
        {
            System.out.println("Caught Expected Exception");
        }
		finally {
			if (trans != null)
				trans.close();
		}

        trans.close();
   
		System.out.println("\nTest: insertTable2");

		try {
			trans = ndb.startTransaction();

			NdbOperation op1 = trans.getInsertOperation(TABLE2);

			op1.setString( key1,  rowTable2    );
			op1.setInt(    key2,  10          );
			op1.setString( data1, "mynewvalue");
			op1.setInt(    data2, 42          );

			NdbOperation op2 = trans.getInsertOperation(TABLE2);

			op2.setString( key1,  rowTable2      );
			op2.setInt(    key2,  11            );
			op2.setString( data1, "myothervalue");
			op2.setInt(    data2, 5464          );

			trans.executeNoCommit();


			trans.executeCommit();
        }
		finally {
			if (trans != null)
				trans.close();
		}




		System.out.println("\nTest: insertTable3");


		try {
		trans = ndb.startTransaction();

		op = trans.getInsertOperation(TABLE3);

		op.setString( key1, rowTable3 );

		Timestamp timestamp = new Timestamp( System.currentTimeMillis() );
		op.setTimestamp( data1, timestamp );


		op.setString( data2, "T" );

		byte[] smallbytes = new byte[60];
		for( int idx=0; idx<smallbytes.length; idx++ )
		{
			smallbytes[idx] = (byte)(idx + 10);
		}
		op.setBytes( data3, smallbytes );



        byte largeBytes[] = new byte[ TEST_BLOB_SIZE ];
        for( int idx=0; idx<largeBytes.length; idx++ )
        {
            largeBytes[idx] = (byte)(48+idx%10);
        }

        NdbBlob d4Blob = op.getBlobHandle(data4);
        d4Blob.setValue( largeBytes );


        StringBuilder text = new StringBuilder();
        String testString = rowTable3 + ":Red Lorry, Yellow Lorry.";

        while( text.length() < TEST_BLOB_SIZE )
        {
            text.append(testString);
        }

        NdbBlob d5Blob = op.getBlobHandle( data5 );
        d5Blob.setValue( text.toString().getBytes("Latin1") );



		long longnum = 0x1234567890aabbccL;
		op.setLong( data6, longnum );

		System.out.println( timestamp.getTime() + " " + Long.toHexString(timestamp.getTime()));
		System.out.println( longnum + " " + Long.toHexString(longnum) );



        StringBuilder text2 = new StringBuilder();
        String testString2 = rowTable2+": Peter piper picked a peck of pickled pepper.";

        while( text2.length() < TEST_BLOB_SIZE )
        {
            text2.append(testString2);
        }

        System.out.println("d7 String length:"+text2.length());

        NdbBlob d7Blob = op.getBlobHandle( data7 );
        d7Blob.setValue( text2.toString().getBytes("UTF8") );



        trans.executeNoCommit();

		trans.executeCommit();
		}
		finally {
			if (trans != null)
				trans.close();
		}


		System.out.println("\nTest: readPrimary from Table1");

		try {
			trans = ndb.startTransaction();
			trans.setOperationLockMode(LockMode.LM_Exclusive);
			NdbOperation op1 = trans.getSelectOperation(TABLE1);
			
			op1.equalString(key1,rowTable1);

			op1.getValue(key1);
			op1.getValue(data1);
			op1.getValue(data2);

			rs = op1.resultData();

			trans.executeNoCommit();

			printTable1((NdbResultSet)rs);

			trans.executeCommit();
	    }
		finally {
			if (trans != null)
				trans.close();
		}

	
		System.out.println("\nTest: readPrimary from Table2");

		try {
			trans = ndb.startTransaction();

			trans.setOperationLockMode(LockMode.LM_Exclusive);
			NdbOperation op1 = trans.getSelectOperation(TABLE2);

			op1.equalString(key1,rowTable2);
			op1.equalInt(key2,10);

			op1.getValue(key1);
			op1.getValue(key2);
			op1.getValue(data1);
			op1.getValue(data2);

			rs = op1.resultData();

			trans.executeNoCommit();

			printTable2((NdbResultSet)rs);

			trans.executeCommit();

	    }
		finally {
			if (trans != null)
				trans.close();
		}

        System.out.println("\nTest: Read Table3");

        try {
	        trans = ndb.startTransaction();

	        trans.setOperationLockMode(LockMode.LM_Exclusive);
	        NdbOperation op1 = trans.getSelectOperation(TABLE3);

	        op1.equalString( key1, rowTable3);
	        op1.getValue(data3);

	        rs = op1.resultData();

	        trans.executeNoCommit();

	        if( rs.next() )
	        {
	            byte[] bytes = rs.getBytes(data3);

	            System.out.println("Bytes:"+convertToHexString(bytes));
	        }

	        trans.executeCommit();
	    }
		finally {
			if (trans != null)
				trans.close();
		}


		System.out.println("\nTest: IndexScan using Primary for Table2");

		try {
			trans = ndb.startTransaction();

			trans.setOperationLockMode(LockMode.LM_Exclusive);
			
			NdbIndexScanOperation op1 = trans.getSelectIndexScanOperation(PRIMARYKEYINDEXNAME, TABLE2);


			op1.equalString(key1,rowTable2);

			op1.getValue(key1);
			op1.getValue(key2);
			op1.getValue(data1);
			op1.getValue(data2);

			rs = op1.resultData();

			trans.executeNoCommit();

			printTable2((NdbResultSet)rs);

			trans.executeCommit();
	    }
		finally {
			if (trans != null)
				trans.close();
		}

		System.out.println("\nTest: scanAll Table1");

		try {
			trans = ndb.startTransaction();
			
			trans.setOperationLockMode(LockMode.LM_Exclusive);

			NdbScanOperation op1 = trans.getSelectScanOperation(TABLE1);

			op1.getValue(key1);
			op1.getValue(data1);
			op1.getValue(data2);

			rs = op1.resultData();

			trans.executeNoCommit();

			printTable1((NdbResultSet)rs);

			trans.executeCommit();
	    }
		finally {
			if (trans != null)
				trans.close();
		}

		System.out.println("\nTest: scanAll Table2");

		try {
			trans = ndb.startTransaction();

			trans.setOperationLockMode(LockMode.LM_Exclusive);
			
			NdbScanOperation op1 = trans.getSelectScanOperation(TABLE2);

			op1.getValue(key1);
			op1.getValue(key2);
			op1.getValue(data1);
			op1.getValue(data2);

			rs = op1.resultData();

			trans.executeNoCommit();

			printTable2((NdbResultSet)rs);

			trans.executeCommit();
	    }
		finally {
			if (trans != null)
				trans.close();
		}

		System.out.println("\nTest: scanAll Table3");

		try {
			trans = ndb.startTransaction();

			trans.setOperationLockMode(LockMode.LM_Exclusive);
			
			NdbIndexScanOperation op1 = trans.getSelectIndexScanOperation(PRIMARYKEYINDEXNAME, TABLE3);

			op1.getValue(key1);
			op1.getValue(data1);
			op1.getValue(data2);
			op1.getValue(data3);
			op1.getBlob(data4,TEST_BLOB_SIZE+1000);
			op1.getBlob(data5,TEST_BLOB_SIZE+1000);
			op1.getValue(data6);
	        op1.getBlob(data7,TEST_BLOB_SIZE+1000);

			rs = op1.resultData();

			trans.executeNoCommit();

			System.out.println("    Table3 results");

			while( rs.next() )
			{
				// table3:
				// d1 TIMESTAMP,
				// d2 CHAR(1),
				// d3 VARBINARY(100),
				// d4 BLOB,
				// d5 TEXT,
				// d6 BIGINT

				String    k1 = rs.getString(key1);
				Timestamp d1 = rs.getTimestamp(data1 );
				String    d2 = rs.getString(   data2 );
				byte[]    d3 = rs.getBytes(    data3 );
				long      d6 = rs.getLong(     data6 );

	            System.out.println( "Timestamp long: " + Long.toHexString(d1.getTime()));

				NdbBlob   d4 = (NdbBlob)rs.getBlob(data4);

	            NdbBlob   d5 = (NdbBlob)rs.getBlob(data5);
	            String    d5str = null;

	            try
	            {
	            	// TODO: Fix blob getLength()
	                //int len = d5.getLength().intValue();

	                byte[] d5Bytes = d5.getData();

	                System.out.println("bloblength:"+d5.getLength()+" array size:"+d5Bytes.length);

	                d5str = new String(d5Bytes, 0, d5.getLength().intValue(), "Latin1" );
	            }
	            catch (UnsupportedEncodingException e)
	            {
	                // TODO Auto-generated catch block
	                e.printStackTrace();
	            }

	            NdbBlob   d7 = (NdbBlob)rs.getBlob(data7);
	            String    d7str = null;

	            try
	            {
	            	// TODO: Fix blob getLength()
	                //int len = d7.getLength().intValue();

	                byte[] d7Bytes = d7.getData();

	                d7str = new String(d7Bytes, 0, d7.getLength().intValue(), "UTF8" );

	                System.out.println("bloblength:"+d7.getLength()+" array size:"+d7Bytes.length+" string length:"+d7str.length());

	            }
	            catch (UnsupportedEncodingException e)
	            {
	                // TODO Auto-generated catch block
	                e.printStackTrace();
	            }


            	// TODO: Fix blob getLength()
	            int d4len = d4.getLength().intValue();
	            byte[] d4bytes = d4.getData();


				System.out.printf("      Row: %-10s '%tc' '%s' \nd3 (VarBinary):'%s' \nd4(blob)'%s' \nd5(Text)'%s' \nd6(long)'%d' \nd7(Text UTF8)'%s' \n",
	                              k1,
	                              d1,
	                              d2,
	                              convertToHexString( d3 ),
	                              convertToHexString( d4bytes, d4len ),
	                              d5str,
	                              new Long(d6),
	                              d7str
	                              );


			}

			trans.executeCommit();
	    }
		finally {
			if (trans != null)
				trans.close();
		}


		System.out.println("\nTest: updateTable1");

		try {
			trans = ndb.startTransaction();

			op = trans.getUpdateOperation(TABLE1);

			op.equalString( key1, rowTable1 );

			op.setString( data1, "anothervalue");
			op.setInt(    data2, 24            );

			trans.executeNoCommit();

			trans.executeCommit();
	    }
		finally {
			if (trans != null)
				trans.close();
		}


		System.out.println("\nTest: updateTable2");

		try {
			trans = ndb.startTransaction();

			op = trans.getUpdateOperation(TABLE2);
			
			op.equalString( key1, rowTable2 );
			op.equalInt( key2, 10       );

			op.setString( data1, "yetanothervalue");

			trans.executeNoCommit();

			trans.executeCommit();
	    }
		finally {
			if (trans != null)
				trans.close();
		}


		System.out.println("\nTest: deleteTable1");

		try {
			trans = ndb.startTransaction();

			op = trans.getDeleteOperation(TABLE1);

			op.equalString( key1, rowTable1 );

			trans.executeNoCommit();

			trans.executeCommit();
	    }
		finally {
			if (trans != null)
				trans.close();
		}


		System.out.println("\nTest: deleteTable2");

		try {
			trans = ndb.startTransaction();

			trans.setOperationLockMode(LockMode.LM_Exclusive);
			
			NdbIndexScanOperation scanOp = trans.getSelectIndexScanOperation( PRIMARYKEYINDEXNAME, TABLE2);
			
			scanOp.equalString( key1, rowTable2 );

			rs = scanOp.resultData();

			trans.executeNoCommit();


			while( rs.next() )
			{
				rs.deleteRow();
			}

			trans.executeCommit();
	    }
		finally {
			if (trans != null)
				trans.close();
		}


		System.out.println("\nTest: deleteTable3");

		try {
			trans = ndb.startTransaction();

			op = trans.getDeleteOperation(TABLE3);

			/**
			 * TODO: Fix this
			 * BUG: Delete on a row that doesn't exist with
			 * a Blob column causes an error
			 */
			op.equalString( key1, rowTable3 );

			trans.executeNoCommit();

			trans.executeCommit();
	    }
		finally {
			if (trans != null)
				trans.close();
		}

	}

}
